import sys
from typing import List, TypeVar

PluginClass = TypeVar("PluginClass")


if sys.version_info < (3, 10):  # pragma: no cover
    from importlib_metadata import entry_points
else:  # pragma: no cover
    from importlib.metadata import entry_points


def plugins(group_name: str) -> List[PluginClass]:
    """List of plugin classes by group name."""
    return [entry_point.load() for entry_point in entry_points(group=group_name)]



import logging
from concurrent.futures import ThreadPoolExecutor
from typing import Any, cast

from rdflib.term import Node
from rich.console import RenderableType
from textual.app import App, ComposeResult
from textual.css.query import NoMatches
from textual.widgets import Footer, Header

from iolanta.facets.textual_browser.page_switcher import (
    ConsoleSwitcher,
    DevConsole,
    PageSwitcher,
)
from iolanta.iolanta import Iolanta

POPUP_TIMEOUT = 30  # seconds


class DevConsoleHandler(logging.Handler):
    """Pipe log output → dev console."""

    def __init__(self, console: DevConsole, level=logging.NOTSET) -> None:
        """Set parameters."""
        self.console = console
        super().__init__(level=level)

    def emit(self, record: logging.LogRecord) -> None:
        """Write a message when invoked by `logging`."""
        message = self.format(record)
        self.console.write(message)


def _log_message_to_dev_console(app: App[None]):
    """Log a message to the dev console."""

    def log_message_to_dev_console(message: str):  # noqa: WPS430
        try:
            app.query_one(DevConsole).write(message)
        except NoMatches:
            return

    return log_message_to_dev_console


class IolantaBrowser(App[None]):  # noqa: WPS214, WPS230
    """Browse Linked Data."""

    def __init__(self, iolanta: Iolanta, this: Node):
        """Set up parameters for the browser."""
        self.iolanta = iolanta
        self.this = this
        self.renderers = ThreadPoolExecutor()
        super().__init__()

    BINDINGS = [  # noqa: WPS115
        ("q", "quit", "Quit"),
    ]

    def compose(self) -> ComposeResult:
        """Compose widgets."""
        yield Header(icon="👁️")
        yield Footer()
        yield ConsoleSwitcher()  # type: ignore[no-untyped-call]

    def on_mount(self):
        """Set title."""
        self.title = "Iolanta"

        logging.basicConfig(
            level=logging.INFO,
            handlers=[
                DevConsoleHandler(
                    console=self.query_one(DevConsole),
                    level=logging.INFO,
                ),
            ],
            force=True,
        )

        # Disable stderr logging, to not break the TUI.
        # Remove only the stderr handler, keep file handler (loguru API)
        loguru_logger = cast(Any, self.iolanta.logger)
        logger_core = loguru_logger._core
        for handler_id in list(logger_core.handlers.keys()):
            log_handler = logger_core.handlers[handler_id]
            if hasattr(log_handler, "sink") and str(log_handler.sink) == "<stderr>":
                loguru_logger.remove(handler_id)

        # Log to the dev console.
        loguru_logger.add(
            _log_message_to_dev_console(self),
            level="INFO",
            format="{time} {level} {message}",
        )

        loguru_logger.add(
            lambda msg: self.notify(
                msg,
                severity="warning",
                timeout=POPUP_TIMEOUT,
            ),
            level="WARNING",
            format="{message}",
        )

    def action_goto(
        self,
        destination: Node | str,
        facet_iri: str | None = None,
    ) -> None:
        """Go to an IRI."""
        self.query_one(PageSwitcher).action_goto(destination, facet_iri)

    def dev_console_log(self, renderable: RenderableType | object):
        """Print a renderable to the dev console."""
        self.query_one(DevConsole).write(renderable)



from __future__ import annotations

import inspect
from abc import ABC
from dataclasses import dataclass, field
from logging import Logger
from pathlib import Path
from typing import TYPE_CHECKING, Optional

from iolanta.models import TripleTemplate

if TYPE_CHECKING:
    from typer import Typer  # noqa: F401

    from iolanta.iolanta import Iolanta


@dataclass
class Plugin(ABC):
    """Base Iolanta plugin."""

    iolanta: Iolanta = field(repr=False)

    @property
    def logger(self) -> Logger:
        return self.iolanta.logger

    @property
    def files_directory(self) -> Path:
        return Path(inspect.getfile(self.__class__)).parent / "data"

    @property
    def context_path(self) -> Optional[Path]:
        context_path = self.files_directory / "context.yaml"
        if context_path.is_file():
            return context_path

    @property
    def data_files(self) -> Path:
        """Directory containing plugin data files."""
        return self.files_directory

    def retrieve_triple(self, triple_template: TripleTemplate) -> None:
        """Save datasets which might describe the given node into project."""
